#!/usr/bin/env python

#
# Copyright 2023 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''
Main validator class
'''
# from media.validation.tests.media.internal import MediaTests
from media.validation.core.status import Tray
from media.validation.suites.default import DefaultValidatorSuite


class Validator():
    '''
    Main validator class that loads all media objects,
    and runs all tests.
    '''
    def __init__(self):
        self.petre = []
        self.suite = None

    def set_suite(self, in_suite=None):
        '''
        Set the main test class used in the tests.
        '''
        if not in_suite:
            self.suite = DefaultValidatorSuite()
        else:
            self.suite = in_suite

    def load_media(self, in_media, in_level=5):
        '''
        Load all of the media classes to be tested.
        '''
        for media in in_media:
            self.petre.append(Tray(media, in_level))

    def run_tests(self):
        '''
        Run tests against media objects, then
        calculate final scores.
        '''
        # results = []
        for dish in self.petre:
            self.suite.run(dish, dish.level)
        for dish in self.petre:
            dish.status.calculate()
        return self.petre
